import 'package:flutter/material.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_material.dart';

class MaterialBadge extends StatelessWidget {
  final String text;
  final String caption;
  final bool isNotScheduled;
  final bool isOnTime;
  final bool isDelayed;

  const MaterialBadge({
    required this.text,
    required this.caption,
    this.isNotScheduled = false,
    this.isOnTime = false,
    this.isDelayed = false,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    Color foregroundColor = Colors.white70;
    Color? backgroundColor;

    if (isNotScheduled) {
      foregroundColor = Colors.orange.shade300;
      backgroundColor = Colors.orange.shade900.withOpacity(0.3);
    }
    else if (isOnTime) {
      foregroundColor = Colors.green.shade300;
      backgroundColor = Colors.green.shade900.withOpacity(0.3);
    }
    else if (isDelayed) {
      foregroundColor = Colors.red.shade300;
      backgroundColor = Colors.red.shade900.withOpacity(0.3);
    }

    return Padding(
      padding: const EdgeInsets.all(8),
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(10),
          border: Border.all(
            width: 2,
            color: foregroundColor,
          ),
          color: backgroundColor,
        ),
        width: isSmallScreen(context) ? 42 : 48,
        height: isSmallScreen(context) ? 42 : 48,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Expanded(
              child: Center(
                child: Text(
                  text,
                  style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                    fontSize: isSmallScreen(context) ? 16 : 20,
                    fontWeight: MediaQuery.of(context).boldText ? FontWeight.w400 : FontWeight.w200,
                    color: MediaQuery.of(context).boldText ? Colors.white70 : foregroundColor,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
            ),
            Text(
              caption,
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                fontSize: 10,
                color: MediaQuery.of(context).boldText ? Colors.white70 : foregroundColor,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
