import 'package:flutter/widgets.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/station_arrdep_page/select_station/select_station_cupertino.dart';
import 'package:info_tren/pages/station_arrdep_page/select_station/select_station_fluent.dart';
import 'package:info_tren/pages/station_arrdep_page/select_station/select_station_material.dart';
import 'package:info_tren/pages/station_arrdep_page/view_station/view_station.dart';
import 'package:info_tren/providers.dart';
import 'package:info_tren/api/stations.dart' as api_stations;

class SelectStationPage extends ConsumerWidget {
  const SelectStationPage({ super.key });

  static String routeName = '/stationArrDep/selectStation';
  
  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final uiDesign = ref.watch(uiDesignProvider);
    switch (uiDesign) {
      case UiDesign.MATERIAL:
        return const SelectStationPageMaterial();
      case UiDesign.CUPERTINO:
        return const SelectStationPageCupertino();
      case UiDesign.FLUENT:
        return const SelectStationPageFluent();
      default:
        throw UnmatchedUiDesignException(uiDesign);
    }
  }
}

abstract class SelectStationPageShared extends StatefulWidget {
  const SelectStationPageShared({super.key});
}

abstract class SelectStationPageState extends State<SelectStationPageShared> {
  static const pageTitle = 'Plecări/sosiri stație';
  static const textFieldLabel = 'Numele stației';
  static const roToEn = {
    'ă': 'a',
    'â': 'a',
    'î': 'i',
    'ș': 's',
    'ț': 't',
  };

  final textEditingController = TextEditingController();

  List<String> stations = [];

  List<String> get filteredStations {
    final filter = textEditingController.text
      .trim()
      .toLowerCase()
      .replaceAllMapped(RegExp(r'[ăâîșț]'), (match) => roToEn[match[0]!]!);
    if (filter.isEmpty) {
      return stations;
    }
    
    return stations.where(
      (e) => e
        .toLowerCase()
        .replaceAllMapped(RegExp(r'[ăâîșț]'), (match) { 
          return roToEn[match[0]!]!;
        })
        .contains(filter)
    ).toList(growable: false);
  }

  @override
  void initState() {
    api_stations.stations.then((value) {
      setState(() {
        stations = value.map((e) => e.name).toList(growable: false,);
      });
    });

    super.initState();
  }

  void onTextChanged(String newText) {
    setState(() {});
  }

  void onSuggestionSelected(String suggestion) {
    Navigator.of(context).pushNamed(
      ViewStationPage.routeName, 
      arguments: ViewStationArguments(stationName: suggestion),
    );
  }

}
