import 'package:flutter/widgets.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:info_tren/components/refresh_future_builder.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/station_arrdep_page/view_station/view_station_cupertino.dart';
import 'package:info_tren/pages/station_arrdep_page/view_station/view_station_fluent.dart';
import 'package:info_tren/pages/station_arrdep_page/view_station/view_station_material.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info.dart';
import 'package:info_tren/providers.dart';

class ViewStationPage extends HookConsumerWidget {
  const ViewStationPage({ super.key, });

  static String routeName = '/stationArrDep/viewStation';

  ValueNotifier<ViewStationPageTab> useTab() => useState(ViewStationPageTab.departures);
  
  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final tab = useTab();

    final uiDesign = ref.watch(uiDesignProvider);
    switch (uiDesign) {
      case UiDesign.MATERIAL:
        return ViewStationPageMaterial(
          tab: tab.value,
          setTab: (newTab) => tab.value = newTab,
        );
      case UiDesign.CUPERTINO:
        return ViewStationPageCupertino(
          tab: tab.value,
          setTab: (newTab) => tab.value = newTab,
        );
      case UiDesign.FLUENT:
        return ViewStationPageFluent(
          tab: tab.value,
          setTab: (newTab) => tab.value = newTab,
        );
    }
  }
}

class ViewStationArguments {
  final String stationName;

  const ViewStationArguments({required this.stationName});
}

abstract class ViewStationPageShared extends StatelessWidget {
  static const arrivals = 'Sosiri';
  static const departures = 'Plecări';
  static const loadingText = 'Se încarcă...';
  static const arrivesFrom = 'Sosește de la';
  static const arrivedFrom = 'A sosit de la';
  static const cancelledArrival = 'Anulat - de la';
  static const departsTo = 'Pleacă către';
  static const departedTo = 'A plecat către';
  static const cancelledDeparture = 'Anulat - către';
  static const errorText = 'A apărut o eroare';
  static const retryText = 'Reîncearcă';

  final ViewStationPageTab tab;
  final void Function(ViewStationPageTab) setTab;
  const ViewStationPageShared({required this.tab, required this.setTab, super.key});

  Widget buildContent(BuildContext context, Future Function() refresh, Future Function(Future<StationData> Function() newFutureBuilder) _, RefreshFutureBuilderSnapshot<StationData> snapshot);
  Widget buildStationArrivalItem(BuildContext context, StationArrDep item);
  Widget buildStationDepartureItem(BuildContext context, StationArrDep item);

  void onTrainTapped(BuildContext context, StationTrain train) {
    Navigator.of(context).pushNamed(
      TrainInfo.routeName, 
      arguments: TrainInfoArguments(
        trainNumber: train.number,
        date: train.departureDate,
      ),
    );
  }

  void onTabChange (int index) {
    setTab(ViewStationPageTab.values[index]);
  }

  @override
  Widget build(BuildContext context) {
    return Consumer(
      builder: (context, ref, _) {
        return RefreshFutureBuilderProviderAdapter(
          futureProvider: viewStationDataProvider,
          refresh: () async {
            ref.invalidate(stationDataProvider);
          },
          builder: buildContent,
        );
      }
    );
  }
}

enum ViewStationPageTab {
  arrivals,
  departures,
}
