import 'package:flutter/cupertino.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:info_tren/components/loading/loading.dart';
import 'package:info_tren/components/refresh_future_builder.dart';
import 'package:info_tren/components/sliver_persistent_header_padding.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/station_arrdep_page/view_station/view_station.dart';
import 'package:info_tren/providers.dart';

class ViewStationPageCupertino extends ViewStationPageShared {
  const ViewStationPageCupertino({super.key, required super.tab, required super.setTab});

  @override
  Widget buildContent(BuildContext context, Future Function() refresh, Future Function(Future<StationData> Function()) _, RefreshFutureBuilderSnapshot<StationData> snapshot) {
    return CupertinoPageScaffold(
      navigationBar: CupertinoNavigationBar(
        middle: Consumer(
          builder: (context, ref, _) {
            final stationName = ref.watch(viewStationArgumentsProvider.select((value) => value.stationName));
            return Text(snapshot.hasData ? snapshot.data!.stationName : stationName);
          }
        ),
      ),
      child: snapshot.hasData ? CupertinoTabScaffold(
        tabBar: CupertinoTabBar(
          items: const [
            BottomNavigationBarItem(
              icon: Icon(CupertinoIcons.arrow_down),
              label: ViewStationPageShared.arrivals,
            ),
            BottomNavigationBarItem(
              icon: Icon(CupertinoIcons.arrow_up),
              label: ViewStationPageShared.departures,
            ),
          ],
          onTap: onTabChange,
          currentIndex: tab.index,
        ),
        tabBuilder: (context, index) {
          final topPadding = MediaQuery.of(context).padding.top;
          return NestedScrollView(
            headerSliverBuilder: (context, _) {
              return [SliverPersistentHeaderPadding(maxHeight: topPadding)];
            },
            body: CustomScrollView(
              slivers: [
                SliverList(
                  delegate: SliverChildBuilderDelegate(
                    (context, index) {
                      return tab == ViewStationPageTab.arrivals ? buildStationArrivalItem(context, snapshot.data!.arrivals![index]) : buildStationDepartureItem(context, snapshot.data!.departures![index]);
                    },
                    childCount: tab == ViewStationPageTab.arrivals ? snapshot.data!.arrivals?.length ?? 0 : snapshot.data!.departures?.length ?? 0,
                  ),
                ),
              ],
            ),
          );
        },
      ) : snapshot.state == RefreshFutureBuilderState.waiting ? const Loading(text: ViewStationPageShared.loadingText,) : Container(),
    );
  }

  @override
  Widget buildStationArrivalItem(BuildContext context, StationArrDep item) {
    return GestureDetector(
      onTap: () => onTrainTapped(context, item.train),
      child: CupertinoFormRow(
        prefix: Text.rich(
          TextSpan(
            children: [
              TextSpan(
                text: item.train.rank,
                style: TextStyle(
                  color: item.train.rank.startsWith('IR') ? const Color.fromARGB(255, 255, 0, 0) : null,
                ),
              ),
              const TextSpan(text: ' '),
              TextSpan(text: item.train.number,),
            ],
          ),
        ),
        helper: Text.rich(
          TextSpan(
            children: [
              const TextSpan(text: ViewStationPageShared.arrivesFrom),
              const TextSpan(text: ' '),
              TextSpan(text: item.train.terminus),
            ],
          ),
        ),
        child: Text('${item.time.toLocal().hour.toString().padLeft(2, '0')}:${item.time.toLocal().minute.toString().padLeft(2, '0')}'),
      ),
    );
  }

  @override
  Widget buildStationDepartureItem(BuildContext context, StationArrDep item) {
    return GestureDetector(
      onTap: () => onTrainTapped(context, item.train),
      child: CupertinoFormRow(
        prefix: Text.rich(
          TextSpan(
            children: [
              TextSpan(
                text: item.train.rank,
                style: TextStyle(
                  color: item.train.rank.startsWith('IR') ? const Color.fromARGB(255, 255, 0, 0) : null,
                ),
              ),
              const TextSpan(text: ' '),
              TextSpan(text: item.train.number,),
            ],
          ),
        ),
        helper: Text.rich(
          TextSpan(
            children: [
              const TextSpan(text: ViewStationPageShared.departsTo),
              const TextSpan(text: ' '),
              TextSpan(text: item.train.terminus),
            ],
          ),
        ),
        child: Text('${item.time.toLocal().hour.toString().padLeft(2, '0')}:${item.time.toLocal().minute.toString().padLeft(2, '0')}'),
      ),
    );
  }
}
