import 'dart:math';
import 'dart:ui';

import 'package:fluent_ui/fluent_ui.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:info_tren/components/badge/badge.dart';
import 'package:info_tren/components/loading/loading.dart';
import 'package:info_tren/components/refresh_future_builder.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/station_arrdep_page/view_station/view_station.dart';
import 'package:info_tren/providers.dart';

class ViewStationPageFluent extends ViewStationPageShared {
  const ViewStationPageFluent({super.key, required super.tab, required super.setTab});

  @override
  Widget buildContent(BuildContext context, Future Function() refresh, Future Function(Future<StationData> Function()) _, RefreshFutureBuilderSnapshot<StationData> snapshot) {
    return NavigationView(
      appBar: NavigationAppBar(
        title: Consumer(
          builder: (context, ref, _) {
            final stationName = ref.watch(viewStationArgumentsProvider.select((value) => value.stationName));
            return Text(snapshot.hasData ? snapshot.data!.stationName : stationName);
          }
        ),
      ),
      content: snapshot.state == RefreshFutureBuilderState.waiting || snapshot.state == RefreshFutureBuilderState.refreshError
          ? const Loading(text: ViewStationPageShared.loadingText,)
          : snapshot.state == RefreshFutureBuilderState.error
          ? Padding(
            padding: const EdgeInsets.all(8.0),
            child: Center(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  Icon(
                    FluentIcons.error,
                    size: 32,
                    color: Colors.red.normal,
                  ),
                  const Text(
                    ViewStationPageShared.errorText,
                    style: TextStyle(
                      inherit: true,
                      fontSize: 32,
                    ),
                  ),
                  Text(
                    snapshot.error.toString(),
                    style: FluentTheme.of(context).typography.subtitle,
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Button(
                      onPressed: () {
                        refresh();
                      },
                      child: const Text(ViewStationPageShared.retryText),
                    ),
                  ),
                ],
              ),
            ),
          )
          : null,
      pane: snapshot.hasData ? NavigationPane(
        onChanged: onTabChange,
        selected: tab.index,
        items: [
          PaneItem(
            body: CustomScrollView(
              slivers: [
                SliverToBoxAdapter(child: SafeArea(left: false, bottom: false, right: false,child: Container(),),),
                SliverList(
                  delegate: SliverChildBuilderDelegate(
                        (context, index) {
                      return tab == ViewStationPageTab.arrivals ? buildStationArrivalItem(context, snapshot.data!.arrivals![index]) : buildStationDepartureItem(context, snapshot.data!.departures![index]);
                    },
                    childCount: tab == ViewStationPageTab.arrivals ? snapshot.data!.arrivals?.length ?? 0 : snapshot.data!.departures?.length ?? 0,
                  ),
                ),
              ],
            ),
            title: const Text(ViewStationPageShared.arrivals),
            icon: const Icon(FluentIcons.arrow_down_right8),
          ),
          PaneItem(
            body: CustomScrollView(
              slivers: [
                SliverToBoxAdapter(child: SafeArea(left: false, bottom: false, right: false,child: Container(),),),
                SliverList(
                  delegate: SliverChildBuilderDelegate(
                        (context, index) {
                      return tab == ViewStationPageTab.arrivals ? buildStationArrivalItem(context, snapshot.data!.arrivals![index]) : buildStationDepartureItem(context, snapshot.data!.departures![index]);
                    },
                    childCount: tab == ViewStationPageTab.arrivals ? snapshot.data!.arrivals?.length ?? 0 : snapshot.data!.departures?.length ?? 0,
                  ),
                ),
              ],
            ),
            title: const Text(ViewStationPageShared.departures),
            icon: const Icon(FluentIcons.arrow_up_right8),
          ),
        ],
      ) : null,
      // bottomNavigationBar: snapshot.hasData ? BottomNavigationBar(
      //   items: const [
      //     BottomNavigationBarItem(
      //       icon: Icon(Icons.arrow_downward),
      //       label: ViewStationPageShared.arrivals,
      //     ),
      //     BottomNavigationBarItem(
      //       icon: Icon(Icons.arrow_upward),
      //       label: ViewStationPageShared.departures,
      //     ),
      //   ],
      //   currentIndex: tab.index,
      //   onTap: onTabChange,
      // ) : null,
    );
  }

  Widget buildStationItem(BuildContext context, StationArrDep item, {required bool arrival}) {
    return HoverButton(
      onPressed: () => onTrainTapped(context, item.train),
      builder: (context, states) {
        return Container(
          color: item.status.cancelled
              ? Colors.red.withAlpha(100)
              : states.isPressing
              ? FluentTheme.of(context).scaffoldBackgroundColor
              : states.isHovering
              ? FluentTheme.of(context).inactiveBackgroundColor
              : null,
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Padding(
                padding: const EdgeInsets.all(8),
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Text(
                      '${item.time.toLocal().hour.toString().padLeft(2, '0')}:${item.time.toLocal().minute.toString().padLeft(2, '0')}',
                      style: TextStyle(
                        inherit: true,
                        fontFeatures: const [
                          FontFeature.tabularFigures(),
                        ],
                        decoration: item.status.cancelled || item.status.delay != 0 ? TextDecoration.lineThrough : null,
                        fontSize: item.status.delay != 0 ? 12 : null,
                      ),
                    ),
                    if (item.status.delay != 0) Builder(
                      builder: (context) {
                        final newTime = item.time.add(Duration(minutes: item.status.delay));
                        final delay = item.status.delay > 0;

                        return Text(
                          '${newTime.toLocal().hour.toString().padLeft(2, '0')}:${newTime.toLocal().minute.toString().padLeft(2, '0')}',
                          style: TextStyle(
                            inherit: true,
                            fontFeatures: const [
                              FontFeature.tabularFigures(),
                            ],
                            color: delay ? Colors.red : Colors.green,
                          ),
                        );
                      },
                    ),
                  ],
                ),
              ),
              Expanded(
                child: IgnorePointer(
                  child: ListTile(
                    // isThreeLine: item.status.delay != 0,
                    title: Text.rich(
                      TextSpan(
                        children: [
                          TextSpan(
                            text: item.train.rank,
                            style: TextStyle(
                              color: item.train.rank.startsWith('IR') ? const Color.fromARGB(255, 255, 0, 0) : null,
                            ),
                          ),
                          const TextSpan(text: ' '),
                          TextSpan(text: item.train.number,),
                        ],
                      ),
                    ),
                    subtitle: Text.rich(
                      TextSpan(
                        children: [
                          TextSpan(
                              text: item.status.cancelled
                                  ? (arrival ? ViewStationPageShared.cancelledArrival : ViewStationPageShared.cancelledDeparture)
                                  : item.time.add(Duration(minutes: max(0, item.status.delay))).compareTo(DateTime.now()) < 0
                                  ? (arrival ? ViewStationPageShared.arrivedFrom : ViewStationPageShared.departedTo)
                                  : (arrival ? ViewStationPageShared.arrivesFrom : ViewStationPageShared.departsTo)
                          ),
                          const TextSpan(text: ' '),
                          TextSpan(text: item.train.terminus),
                          if (item.status.delay != 0) ...[
                            const TextSpan(text: '\n'),
                            if (item.status.delay.abs() >= 60) ...[
                              TextSpan(text: (item.status.delay.abs() ~/ 60).toString()),
                              TextSpan(text: item.status.delay.abs() >= 120 ? ' ore' : ' oră'),
                              if (item.status.delay.abs() % 60 != 0)
                                const TextSpan(text: ' și '),
                            ],
                            TextSpan(text: (item.status.delay.abs() % 60).toString()),
                            TextSpan(text: item.status.delay.abs() > 1 ? ' minute' : ' minut'),
                            const TextSpan(text: ' '),
                            if (item.status.delay > 0)
                              TextSpan(
                                text: 'întârziere',
                                style: TextStyle(
                                  inherit: true,
                                  color: Colors.red,
                                ),
                              )
                            else
                              TextSpan(
                                text: 'mai devreme',
                                style: TextStyle(
                                  inherit: true,
                                  color: Colors.green,
                                ),
                              ),
                          ],
                        ],
                      ),
                    ),
                  ),
                ),
              ),
              if (item.status.platform != null)
                IntrinsicHeight(
                  child: AspectRatio(
                    aspectRatio: 1,
                    child: Badge(
                      text: item.status.platform!,
                      caption: 'Linia',
                      isOnTime: item.status.real && item.status.delay <= 0,
                      isDelayed: item.status.real && item.status.delay > 0,
                    ),
                  ),
                ),
            ],
          ),
        );
      },
    );
  }

  @override
  Widget buildStationArrivalItem(BuildContext context, StationArrDep item) {
    return buildStationItem(context, item, arrival: true);
  }

  @override
  Widget buildStationDepartureItem(BuildContext context, StationArrDep item) {
    return buildStationItem(context, item, arrival: false);
  }
}
