
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:info_tren/components/select_train_suggestions/select_train_suggestions.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/pages/train_info_page/select_train/select_train_cupertino.dart';
import 'package:info_tren/pages/train_info_page/select_train/select_train_fluent.dart';
import 'package:info_tren/pages/train_info_page/select_train/select_train_material.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info.dart';
import 'package:info_tren/providers.dart';
import 'package:info_tren/api/trains.dart' as api_trains;

typedef TrainSelectedCallback = Function(int trainNumber);

class SelectTrainPage extends ConsumerWidget {
  const SelectTrainPage({super.key});

  static String routeName = "/trainInfo/selectTrain";
  
  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final uiDesign = ref.watch(uiDesignProvider);
    
    switch(uiDesign) {
      case UiDesign.MATERIAL:
        return const SelectTrainPageMaterial();
      case UiDesign.CUPERTINO:
        return const SelectTrainPageCupertino();
      case UiDesign.FLUENT:
        return const SelectTrainPageFluent();
      default:
        throw UnmatchedUiDesignException(uiDesign);
    }
  }
}

abstract class SelectTrainPageShared extends StatefulWidget {
  const SelectTrainPageShared({super.key});

  void onTrainSelected(BuildContext context, String selection) {
    selection = selection.characters.takeWhile((char) => List.generate(10, (i) => i.toString()).contains(char)).join();
    Navigator.of(context).pushNamed(
      TrainInfo.routeName, 
      arguments: TrainInfoArguments(trainNumber: selection),
    );
  }
}

abstract class SelectTrainPageState extends State<SelectTrainPageShared> {
  final String pageTitle = 'Informații despre tren';
  final String textFieldLabel = 'Numărul trenului';

  final trainNoController = TextEditingController();

  List<TrainsResult> trains = [];

  List<TrainsResult> get filteredTrains {
    final filter = trainNoController.text
      .trim()
      .toLowerCase();
    if (filter.isEmpty) {
      return trains;
    }

    final filtered = trains.where((e) => e.number.contains(filter)).toList(growable: false);

    filtered.sort((t1, t2) {
      final posInNum1 = t1.number.indexOf(filter);
      final posInNum2 = t2.number.indexOf(filter);

      if (posInNum1 != posInNum2) {
        return posInNum1.compareTo(posInNum2);
      }

      if (t1.number.length != t2.number.length) {
        return t1.number.length.compareTo(t2.number.length);
      }

      return t1.number.compareTo(t2.number);
    });

    return filtered;
  }

  @override
  void initState() {
    api_trains.trains.then((value) {
      setState(() {
        trains = value;
        trains.sort((t1, t2) {
          if (t1.company != t2.company) {
            return t1.company.compareTo(t2.company);
          }

          if (t1.rank != t2.rank) {
            return t1.rank.compareTo(t2.rank);
          }

          final t1Num = t1.number.padLeft(t2.number.length, '0');
          final t2Num = t2.number.padLeft(t1.number.length, '0');

          return t1Num.compareTo(t2Num);
        });
      });
    });

    super.initState();
  }

  void onTextChanged() {
    setState(() {});
  }

  Widget get suggestionsList => SelectTrainSuggestions(
    choices: filteredTrains,
    onTrainSelected: (trainNumber) => widget.onTrainSelected(context, trainNumber),
    currentInput: trainNoController.text,
    key: ValueKey(trainNoController.text),
  );
}
