import 'package:flutter/material.dart';
import 'package:info_tren/models.dart';
import 'package:info_tren/components/badge/badge.dart';
import 'package:info_tren/pages/train_info_page/view_train/train_info_material.dart';

class DisplayTrainStation extends StatelessWidget {
  final Station station;
  final void Function()? onTap;

  const DisplayTrainStation({required this.station, this.onTap, super.key,});

  @override
  Widget build(BuildContext context) {
    return Card(
      child: InkWell(
        onTap: onTap,
        child: Padding(
          padding: const EdgeInsets.all(2),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              Row(
                mainAxisSize: MainAxisSize.max,
                children: <Widget>[
                  Expanded(
                    flex: 1,
                    child: Align(
                      alignment: Alignment.centerLeft,
                      child: Builder(
                        builder: (context) {
                          final departureStatus = station.departure?.status;
                          final arrivalStatus = station.arrival?.status;
                          int delay;
                          bool real;
                          if (departureStatus == null) {
                            delay = arrivalStatus?.delay ?? 0;
                            real = arrivalStatus?.real ?? false;
                          }
                          else if (arrivalStatus == null) {
                            delay = departureStatus.delay;
                            real = departureStatus.real;
                          }
                          else {
                            delay = departureStatus.delay;
                            real = departureStatus.real;
                            if (!real && arrivalStatus.real) {
                              delay = arrivalStatus.delay;
                              real = arrivalStatus.real;
                            }
                          }
                                    
                          final isDelayed = delay > 0 && real == true;
                          final isOnTime = delay <= 0 && real == true;
                          const isNotScheduled = false;
                                    
                          return Badge(
                            text: station.km.toString(),
                            caption: 'km',
                            isNotScheduled: isNotScheduled,
                            isDelayed: isDelayed,
                            isOnTime: isOnTime,
                          );
                        }
                      ),
                    ),
                  ),
                  Title(
                    station: station,
                  ),
                  Expanded(
                    flex: 1, 
                    child: (station.platform == null) 
                      ? Container() 
                      : Align(
                        alignment: Alignment.centerRight,
                        child: Badge(text: station.platform!, caption: 'linia',),
                      ),
                  ),
                ],
              ),
              Time(
                station: station,
              ),
              Delay(
                station: station,
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class Title extends StatelessWidget {
  final Station station;

  const Title({
    required this.station,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return  Text(
      station.name,
      style: Theme.of(context).textTheme.bodyMedium?.copyWith(
        fontSize: isSmallScreen(context) ? 18 : 22,
        fontWeight: MediaQuery.of(context).boldText ? FontWeight.w500 : FontWeight.w300,
        // fontStyle: items[1] == "ONI" ? FontStyle.italic : FontStyle.normal,
      ),
      textAlign: TextAlign.center,
    );
  }
}

class Time extends StatelessWidget {
  final Station station;

  const Time({
    required this.station,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    if (station.arrival == null) {
      // Plecare
      return DepartureTime(
        station: station,
        firstStation: true,
      );
    }

    if (station.departure == null) {
      // Sosire
      return ArrivalTime(
        station: station,
        finalStation: true,
      );
    }

    return Row(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Text(
          "→",
          style: Theme.of(context).textTheme.bodyMedium?.copyWith(
            fontSize: isSmallScreen(context) ? 18 : 22,
          ),
        ),
        Container(width: 2,),
        ArrivalTime(station: station,),
        Expanded(child: Container(),),
        StopTime(station: station,),
        Expanded(child: Container(),),
        DepartureTime(station: station,),
        Container(width: 2,),
        Text(
          "→",
          style: Theme.of(context).textTheme.bodyMedium?.copyWith(
            fontSize: isSmallScreen(context) ? 18 : 22,
          ),
        ),
      ],
    );
  }
}

class ArrivalTime extends StatelessWidget {
  final Station station;
  final bool finalStation;

  const ArrivalTime({
    required this.station,
    this.finalStation = false,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    if (station.arrival == null) {
      return Container();
    }
    if (finalStation) {
      return Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Text(
            "→",
            style: Theme.of(context).textTheme.bodyMedium?.copyWith(
              fontSize: isSmallScreen(context) ? 18 : 22,
            ),
          ),
          Container(width: 2,),
          const Text("sosire la "),
          ArrivalTime(station: station,),
          Expanded(child: Container(),),
        ],
      );
    }
    else {
      final delay = station.arrival!.status?.delay ?? 0;
      final time = station.arrival!.scheduleTime.toLocal();

      if (delay == 0) {
        return Text("${time.hour.toString().padLeft(2, '0')}:${time.minute.toString().padLeft(2, '0')}");
      }
      else if (delay > 0) {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                color: Colors.red.shade300,
              ),
            ),
          ],
        );
      }
      else {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                color: Colors.green.shade300,
              ),
            ),
          ],
        );
      }
    }
  }
}

class StopTime extends StatelessWidget {
  final Station station;

  const StopTime({
    required this.station,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        const Text(
          "staționează pentru",
          textAlign: TextAlign.center,
        ),
        Builder(
          builder: (context) {
            int stopsForInt = station.stoppingTime!;
            bool minutes = false;
            if (stopsForInt >= 60) {
              stopsForInt ~/= 60;
              minutes = true;
            }
            if (stopsForInt == 1) {
              return Text(
                "1 ${minutes ? 'minut' : 'secundă'}",
                textAlign: TextAlign.center,
              );
            }
            else if (stopsForInt < 20) {
              return Text(
                "$stopsForInt ${minutes ? 'minute' : 'secunde'}",
                textAlign: TextAlign.center,
              );
            }
            else {
              return Text(
                "$stopsForInt de ${minutes ? 'minute' : 'secunde'}",
                textAlign: TextAlign.center,
              );
            }
          },
        )
      ],
    );
  }
}

class DepartureTime extends StatelessWidget {
  final Station station;
  final bool firstStation;

  const DepartureTime({
    required this.station,
    this.firstStation = false,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    if (station.departure == null) {
      return Container();
    }
    if (firstStation) {
      return Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Expanded(child: Container(),),
          const Text("plecare la "),
          DepartureTime(station: station,),
          Container(width: 2,),
          Text(
            "→",
            style: Theme.of(context).textTheme.bodyMedium?.copyWith(
              fontSize: 22,
            ),
          ),
        ],
      );
    }
    else {
      final delay = station.departure!.status?.delay ?? 0;
      final time = station.departure!.scheduleTime.toLocal();

      if (delay == 0) {
        return Text("${time.hour.toString().padLeft(2, '0')}:${time.minute.toString().padLeft(2, '0')}");
      }
      else if (delay > 0) {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                color: Colors.red.shade300,
              ),
            ),
          ],
        );
      }
      else {
        final oldDate = time;
        final newDate = oldDate.add(Duration(minutes: delay));

        return Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "${oldDate.hour.toString().padLeft(2, '0')}:${oldDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                decoration: TextDecoration.lineThrough,
              ),
            ),
            Text(
              "${newDate.hour.toString().padLeft(2, '0')}:${newDate.minute.toString().padLeft(2, '0')}",
              style: Theme.of(context).textTheme.bodyMedium?.copyWith(
                color: Colors.green.shade300,
              ),
            ),
          ],
        );
      }
    }
  }
}


class Delay extends StatelessWidget {
  final Station station;

  const Delay({
    required this.station,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    if (station.arrival?.status == null && station.departure?.status == null) {
      return Container();
    }
    var delay = station.arrival?.status?.delay;
    if (station.departure?.status?.real == true) {
      delay = station.departure?.status?.delay;
    }

    if (delay == 0 || delay == null) {
      return Container();
    } else if (delay > 0) {
      return Text(
        "$delay ${delay == 1 ? 'minut' : 'minute'} întârziere",
        style: Theme.of(context).textTheme.bodyMedium?.copyWith(
          color: Colors.red.shade300,
          fontSize: 14,
          fontStyle: FontStyle.italic,
        ),
      );
    }
    else if (delay < 0) {
      return Text(
        "${-delay} ${delay == -1 ? 'minut' : 'minute'} mai devreme",
        style: Theme.of(context).textTheme.bodyMedium?.copyWith(
          color: Colors.green.shade300,
          fontSize: 14,
          fontStyle: FontStyle.italic,
        ),
      );
    }

    return Container();
  }
}
